/*
 *  cave.c
 *
 *  CAVE - Colored Ascii Visual Editor
 *  General routines
 *
 *  Copyright (C) 2002 Michal Miszewski <fixer@irc.pl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 */

#include <curses.h>
#include <stdlib.h>
#include "cave.h"
#include "conffile.h"
#include "color.h"
#include "config.h"
#include "cursor.h"
#include "status.h"
#include "import.h"
#include "cursor.h"
#include "export.h"
#ifdef BANNER
#include "banner.h"
#endif

/* window parameters */
int xsize, ysize;

WINDOW *editw, *helpw;
FILE *rnd;

extern struct config
{
  int defcolor, defbgnd, defattr, defmode, fromzero,
  key_up, key_down, key_left, key_right,
  key_leftborder, key_rightborder, key_upborder, key_downborder,
  key_delch, key_backspc, key_addattribs,
  key_export, key_import, key_color, key_bgnd, key_attrib, key_mode;
} conf;

/* puts character */
void putch(int key)
{
  char ch = key;  /* to remove all attributes */

  mvwaddch(editw, acty, actx, ch | COLOR_PAIR(pair) | avals[attribute]);
  put_cursor(acty, actx);  /* because character hides our cursor */
  wrefresh(editw);
}

/* deletes character */
void rmch(void)
{
  mvwaddch(editw, acty, actx, ' ' | COLOR_PAIR(0) | A_NORMAL);
  put_cursor(acty, actx);
  wrefresh(editw);
}

#ifdef RANDOM

/* opens /dev/urandom */
void open_rnd(void)  // !!! doda gdzie close(rnd); (przy wyczaniu programu)
{
  rnd = fopen(RNDDEV, "r");

  if (rnd == NULL)
  {
    fprintf(stderr, "Error: Can't open /dev/urandom.");
    exit(EXIT_FAILURE);
  }
}

#endif

/* prepares curses and editor screen */
void draw_screen(void)
{  
  initscr();
  start_color();
  getmaxyx(stdscr, ysize, xsize);
  if ((statusw = newwin(1, xsize, ysize - 1, 0)) == NULL) printf("error\n");
  if ((editw = newwin(ysize - 1, xsize, 0, 0)) == NULL) printf("error\n");
  curs_set(FALSE);  /* we have own cursor */
#ifdef BANNER
  draw_banner();
#endif
  noecho();
  put_cursor(acty, actx);
  getmaxyx(editw, ysize, xsize);
  keypad(stdscr, TRUE);
  refresh();
  init_pairs();
  pair = find_pair(color, background);
  clear_status();
}

/* curses shutdown */
void shut_screen(void)
{
  delwin(editw);
  delwin(statusw);
  curs_set(TRUE);
  endwin();
}

/* keyboard interface */
void check_key(int key)
{
  int end = 0;

  keyno = key;
  /* cursor keys */
  if ((end == 0) && (key == conf.key_up)) { move_up(); update_status(); end = 1; }
  if ((end == 0) && (key == conf.key_down)) { move_down(); update_status(); end = 1; }
  if ((end == 0) && (key == conf.key_left)) { move_left(); update_status(); end = 1; }
  if ((end == 0) && (key == conf.key_right)) { move_right(); update_status(); end = 1; }
  if ((end == 0) && (key == conf.key_leftborder)) { mv_cursor(acty, 0); update_status(); end = 1; }
  if ((end == 0) && (key == conf.key_rightborder)) { mv_cursor(acty, xsize - 1); update_status(); end = 1; }
  if ((end == 0) && (key == conf.key_upborder)) { mv_cursor(0, actx); update_status(); end = 1; }
  if ((end == 0) && (key == conf.key_downborder)) { mv_cursor(ysize - 1, actx); update_status(); end = 1; }
  /* character deletion */
  if ((end == 0) && (key == conf.key_delch)) { rmch(); update_status(); end = 1; }
  if ((end == 0) && (key == conf.key_backspc)) { rmch(); update_status(); step_bwd(); end = 1; }
  if ((end == 0) && (key == conf.key_addattribs)) { putch(oldchar); step(); update_status(); end = 1; }  /* only add attributes */
  /* function keys */
  if ((end == 0) && (key == conf.key_export)) { export(); end = 1; }
  if ((end == 0) && (key == conf.key_import)) { import(); end = 1; }
  if ((end == 0) && (key == conf.key_color)) { rotate(&color, NUMCOLORS - 1); pair = find_pair(color, background); end = 1; }
  if ((end == 0) && (key == conf.key_bgnd)) { rotate(&background, NUMCOLORS - 1); pair = find_pair(color, background); end = 1; }
  if ((end == 0) && (key == conf.key_attrib)) { rotate(&attribute, 5); end = 1; }
  if ((end == 0) && (key == conf.key_mode)) { rotate(&mode, NUMMODES - 1); end = 1; }
  /* default */
  if (end == 0) { putch(key); step(); update_status(); }
}

/* init */
int main(int argc, char *argv[])
{
  int key;

#ifdef RANDOM
  open_rnd();
#endif
  read_config();
  color_load_defaults();
  cursor_load_defaults();
  draw_screen();
  wrefresh(editw);
  update_status();
  while(1)
  {
    key = getch();
    if (key != ERR) check_key(key);
  }
}
