/*
 *  import.c
 *
 *  CAVE - Colored Ascii Visual Editor
 *  Image import routines
 *
 *  Copyright (C) 2002 Michal Miszewski <fixer@irc.pl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 */

#include <curses.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include "config.h"
#ifdef HAVE_GLOB
#include <glob.h>
#endif
#include "import.h"
#include "color.h"
#include "cursor.h"
#include "status.h"

FILE *fd;
extern char lastname[500];

struct attribs
{
  short color, background, colorpair;
  chtype attrmask;
};

/* opens a file */
int read_file(char filename[])
{
  int ret = 0;

  fd = fopen(filename, "r");
  if (fd == NULL)  /* error message on status */
  {
    ret = 1;
    status_error();
  }
  return ret;
}

/* parses the ansi code */
void ansicode(char code[], int values[5])
{
  int a, c = 0, d = 0;
  char buf[4] = "";

  for (a = 0; a < 5; a++) values[a] = -1;
  for (a = 0; a < strlen(code); a++)
  {
    if (isdigit(code[a]) != 0)  /* digits */
    {
      if (c < sizeof(buf))
      {
        buf[c] = code[a];
        c++;
      }
    }
    else
    if ((d <= 5) && (strcmp(buf, "") != 0))
    {
      buf[c] = '\0';
      values[d] = atoi(buf);
      strcpy(buf, "");
      c = 0;
      d++;
    }
  }
}

/* zeroes an attribs structure */
void clean_attribs(struct attribs *act)
{
  act->color = 20;
  act->background = 20;
  act->colorpair = 20;
  act->attrmask = A_NORMAL;
}

/* interprets the ansi codes */
void interpret_ansi(char code[], struct attribs *act)
{
  int values[5];
  short color = COLOR_WHITE, bgnd = COLOR_BLACK;
  int a = 0;

  ansicode(code, values);
  switch (code[strlen(code) - 1])
  {
    case 'm':  /* color/attribute change */
    {
      for (a = 0; a < 5; a++)
      if (values[a] != -1)
      {
        if (values[a] == 0) clean_attribs(act);   /* reset code */
	if ((values[a] > 0) && (values[a] <= 7))  /* text attributes */
	{
	  switch (values[a])
	  {
            case 1: { act->attrmask = act->attrmask | A_BOLD; break; }
            case 4: { act->attrmask = act->attrmask | A_UNDERLINE; break; }
	    case 5: { act->attrmask = act->attrmask | A_BLINK; break; }
	    case 7: { act->attrmask = act->attrmask | A_REVERSE; break; }
	    case 8: { act->attrmask = act->attrmask | A_INVIS; break; }
	  }
	}
	/* colour && background */
        if ((values[a] >= 30) && (values[a] <= 37)) color = values[a] - 30;
	if ((values[a] >= 40) && (values[a] <= 47)) bgnd = values[a] - 40;
	if ((color != act->color) || (bgnd != act->background))
	{
	  act->color = color;
	  act->background = bgnd;
	  act->colorpair = find_pair(color, bgnd);
	}
      }
      break;
    }
  }
}

/* image import */
void import_text(void)
{
  int x = 0, y = 0;    /* screen coordinates */
  int a, b;
  char *buf;           /* line buffer */
  char code[15];       /* ansi code buffer */
  struct attribs out;  /* actual character attributes */

  buf = (char *)malloc(sizeof(char) * CBUF);
  if (buf == NULL)
  {
    status_message("Error: Can't allocate memory.");
    return;
  }
  /* startup text attributes */
  {
    out.color = COLOR_WHITE;
    out.background = COLOR_BLACK;
    out.colorpair = find_pair(out.color, out.background);
    out.attrmask = A_NORMAL;
  }
  while (fgets(buf, CBUF, fd) != NULL)
  {
    x = 0;
    for (a = 0; a < CBUF; a++)
    {
      if (buf[a] == '\0') break;  /* end of the string */
      if (buf[a] == '\x1b')       /* escape char */
      {
        strcpy(code, "");
	b = 0;
	a++;
	while (buf[a] != '\0')
	{
	  code[b] = buf[a];
	  /* alphabetical char ends the sequence */
	  if (isalpha(buf[a]) != 0)
	  {
	    code[b + 1] = '\0';
	    interpret_ansi(code, &out);
	    break;
	  }
	  a++;
	  b++;
	}
      }
      else
      {
        mvwaddch(editw, y, x, buf[a] | out.attrmask | COLOR_PAIR(out.colorpair));
        x++;
      }
    }
    y++;
  }
  free(buf);
  wrefresh(editw);
}

void import(void)
{
  char filename[200] = "";
  char realname[200] = "";
#ifdef HAVE_GLOB
  glob_t result;
#endif

  status_input("Image import. File name", "", filename, 60);
  if (strncmp(filename, "", 200) != 0)
  {

#ifdef HAVE_GLOB
    glob(filename, GLOB_TILDE, NULL, &result);
    if (result.gl_pathc > 1) 
    {
      status_message("Wildcards not allowed.");
      globfree(&result);
      return;
    }
    else
    {
      if (result.gl_pathc > 0) strncpy(filename, result.gl_pathv[0], 200);
    }
    globfree(&result);
#endif

    realpath(filename, realname);
    if (read_file(realname) == 0)
    {
      cursor_up();
      import_text();
      fclose(fd);
      strcpy(lastname, filename);
      cursor_down();
    }
  }
}
